/*  bar_chart - bar chart render for stats.
    Copyright (C) 2004  Daniele F.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */

/* For bugs or infos please contact me <geekgnele@tin.it> */

#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include "bar_chart.h"

/* Static prototypes */
static void bar_chart_init       (BarChart *cell);
static void bar_chart_class_init (BarChartClass *class);
static void bar_chart_set_property(GObject *object, guint param_id, const GValue *value, GParamSpec *pspec);
static void bar_chart_get_size   (GtkCellRenderer	*cell,
				      GtkWidget		*widget,
				      GdkRectangle	*rectangle,
				      gint		*x_offset,
				      gint		*y_offset,
				      gint		*width,
				      gint		*height);
static void bar_chart_render	(GtkCellRenderer	*cell,
				      GdkWindow		*window,
				      GtkWidget		*widget,
				      GdkRectangle	*background_area,
				      GdkRectangle	*cell_area,
				      GdkRectangle	*expose_area,
				      guint		flags);

enum {
	PROP_ZERO,
	PROP_VALUE,
	PROP_MAX,
	PROP_SCALED,
};

/****************************************
 *			EXTERNAL INTERFACE			*
 ****************************************/

GtkCellRenderer *bar_chart_new(void)
{
	return (GTK_CELL_RENDERER(g_object_new(BAR_CHART_TYPE, NULL)));
}

/****************************************
 *			INTERNAL FUNCTIONS			*
 ****************************************/


GtkType bar_chart_get_type(void)
{
	static GtkType bar_chart_type = 0;

	if (!bar_chart_type)
	{
		static const GTypeInfo bar_chart_info =
		{
			sizeof (BarChartClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) bar_chart_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (BarChart),
			0,              /* n_preallocs */
			(GInstanceInitFunc) bar_chart_init,
		};

		bar_chart_type = g_type_register_static(GTK_TYPE_CELL_RENDERER, "BarChart", &bar_chart_info, 0);
	}

	return bar_chart_type;
}

static void bar_chart_init(BarChart *bar)
{
	bar->value = 0;
	bar->gc = NULL;
	bar->max = 1;
	bar->scaled = 1;
}

static void bar_chart_finalize(GObject *object)
{
	BarChart *bar = (BarChart *) object;
	
	if (bar->gc)
		g_object_unref(bar->gc);
}

static void bar_chart_class_init(BarChartClass *class)
{
	GObjectClass *object_class = G_OBJECT_CLASS(class);
	GtkCellRendererClass *cell_class = GTK_CELL_RENDERER_CLASS(class);

	object_class->set_property = bar_chart_set_property;
	object_class->finalize = bar_chart_finalize;

	cell_class->get_size = bar_chart_get_size;
	cell_class->render = bar_chart_render;

	g_object_class_install_property(object_class, PROP_VALUE, g_param_spec_double("value", _("Value"), _("The length of the bar segment"), 0, G_MAXDOUBLE, 0, G_PARAM_WRITABLE));

	g_object_class_install_property(object_class, PROP_MAX, g_param_spec_double("max", _("Max"), _("The maximum value on the scale"), 0, G_MAXDOUBLE, 1, G_PARAM_WRITABLE));

	g_object_class_install_property(object_class, PROP_SCALED, g_param_spec_boolean("scaled", _("Scaled"), _("Use max rather than using the full width"), TRUE, G_PARAM_WRITABLE));
}

static void bar_chart_set_property(GObject *object, guint param_id,
				       const GValue *value, GParamSpec *pspec)
{
	BarChart *bar = (BarChart *) object;

	switch (param_id)
	{
		case PROP_VALUE:
			bar->value = g_value_get_double(value);
			break;
		case PROP_MAX:
			bar->max = g_value_get_double(value);
			break;
		case PROP_SCALED:
			bar->scaled = g_value_get_boolean(value);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
			break;
	}
}

static void bar_chart_get_size(GtkCellRenderer *cell,
				   GtkWidget       *widget,
				   GdkRectangle    *cell_area,
				   gint            *x_offset,
				   gint            *y_offset,
				   gint            *width,
				   gint            *height)
{
	if (x_offset)
		*x_offset = 0;
	if (y_offset)
		*y_offset = 0;

	if (width)
		*width = 32;
	if (height)
		*height = 4;
}

static void bar_chart_render(GtkCellRenderer    *cell,
			     GdkWindow		*window,
			     GtkWidget		*widget,
			     GdkRectangle	*background_area,
			     GdkRectangle	*cell_area,
			     GdkRectangle	*expose_area,
			     guint		flags)
{
	BarChart *bar = (BarChart *) cell;
	int y = cell_area->y + 2;
	int h = cell_area->height - 4;
	int v;
	float scale;

	if (bar->scaled)
		scale = cell_area->width / bar->max;
	else
		scale = cell_area->width / (bar->value);

	if (!bar->gc)
	{
		GdkColor c = {0, 0x0, 0xffff, 0x0};
		
		bar->gc = gdk_gc_new(window);
		gdk_gc_set_rgb_fg_color(bar->gc, &c);
	}

	v = bar->value * scale;

	gdk_draw_rectangle(window, bar->gc, TRUE, cell_area->x + 1, y, MAX(0, v - 1), h);

	y -= 1;
	h += 2;

	gdk_draw_rectangle(window, widget->style->black_gc, FALSE, cell_area->x, y, cell_area->width, h - 1);
}
